c        This version is for DOS
c        main.for
c        .................................................................
c        .  Electromagnetic (Light) Scattering by Multilayered Sphere:   .
c        .  Recursive Algorithms          1994, version 3.0              .
c        .       All  Rights  Reserved  By   Professor  Wu ZhengSen      .
c        .  Department of Physics,  Box 274,  Xidian University  710071  .
c        .  Xi'an, Shaanxi,  P. R.  China                                .
c        .  Reference:                                                   .
c        .  Z. S. Wu and Y. P. Wang                                      .
c        .  Electromagnetic scattering for multilayered sphere:          .
c        .  Recursive algorithms,  Radio Scince, Volume 26, Number 6. ,  .
c        .  p1393-1401,1991.                                             .
c        .................................................................
c        .   Main.for is calling subroutine mulsph.for, that calculates  .
c        . scattering efficiencies and Stokes parameters for a multilay- .
c        . ered sphere with given refractive indices and radius within   .
c        . the various regions of the multilayered sphere, the refractive.
c        . index  of surrouning medium, and free space wave length.      .
c        .        (link main+mulsph)                                     .
c        .................................................................
c        .    input:                                                     .
c        .    refmed = (real) refractive index of surrouding medium      .
c        .    wavel  = wavelength in free space                          .
c        .    refel(*) = complex index of refraction, (real,imag)        .
c        .                 (imag. part is positive for absorption)       .
c        .                     for an exp(-iwt) time variation           .
c        .    r = the radii (rj) of the various regions for the          .
c        .              given multilayered sphere in input data-file,    .
c        .                   (unit is same as wavelength) !!             .
c        .    x(j) = the corresponding size parameters of the various    .
c        .            regions for multilayered sphere in input date-file .
c        .                                                               .
c        .         note : format of data is " real, (real,imag)" !!      .
c        .                                                               .
c        .    m = layer number of the multilayered sphere                .
c        .                                                               .
c        .    nang = control case, (=1): only calculate Qext, Qsca,      .
c        .           Qabp, Qback, g        ( i.e.  qe, qs, qba, gg)      .
c        .                             (>1): calculate also S1 and S2    .
c        .                             and control increament of angle   .
c        .    nan = number of scattered angle in 0 and 180,              .
c        .         and number of array for s1,s2, as well as subroutinue .
c        .                                                               .
c        .    x(*) = (krj) correspording size parameters                 .
c        .    yy = the size parameter (xt=krt) correspord to the         .
c        .          exterior radii of the sphere                         .
c        .    ang = scattering angle, degrees                            .
c        .    dang = increament in scattering angle, radian              .
c        .    nstop = int(yy+4.05*yy**.3333+2.0)                         .
c        .    nmax = nstop+15                                            .
c        .                                                               .
c        .    qext, qsca, qabp = the efficiencies for extinction,        .
c        .                      scattering and absorption, respectively  .
c        .    qback = the effiency for backscattering                    .
c        .    g = asymmertry factor ,or the average cosine of the        .
c        .          scattering angle, (<cos(seta)>)                      .
c        .    s1,s2 = scattering amplitude functions                     .
c        .    s11, s12, s33, s34 = scattering matrix elements            .
c        .    se = S11+S12 = (|S2|*|S2|), e.g. SPLL (i-H)                .
c        .    sh = S11-S12 = (|S1|*|S1|), e.g. SPDR (i-V)                .
c        .    SPLL and SPDR = the scattered irradiance per unit incident .
c        .       irradiance, when incident light is polarized parallel   .
c        .       and perpendicular to scattering plane,respectively      .
c        .    S11 = SMEN = (SPLL+SPDR)/2 refers to incident unpolarized  .
c        .                  light                                        .
c        .    pol = -s12/s11 polarized degree, pol > 0, the scattered    .
c        .           light is partially polarized perpendicur to the     .
c        .           scattering plane; pol < 0, refer to parallel        .
c        .                                                               .
c        .    output:                                                    .
c        .    ******.dat is named by operator, it includes all infor-    .
c        .                 mation about input and output                 .
c        .    data-file:                                                 .
cj !!!!!!   The output is NOT logarithmically, J. Rheims, Jan 02, 1996   !!!!!!!                                                       .
c        .         normal.dat = normalized scattering matrix elements    .
c        .                   (s11,pol,s33,s34) vs scattered angle (ang), .
c        .                     where (s11) is denoted logarithmically    .
c        .         inten.dat = scattered intensity (se,sh,smen)          .
c        .                       vs angle   (ang), where (se, sh, smen)  .
c        .                       are denoted logarithmically             .
c        .         ab.dat in subroutine = scattering coefficients (an,bn).
c        .................................................................
     
      program multilayer

      integer layer,j,m,nan,nang, winkel,groesse

      parameter (layer = 100, winkel = 200, groesse = 3000)

cd es gilt die abschaetzung :   ystop=2000+4.*2000**.3333+2. < 2500     

cd layer  anzahl der schichten
cd winkel anzahl der winkelschritte  
cd groesse siehe ystop
      double complex s1(winkel),s2(winkel),sri,rfrel(layer)
      double precision yy,x(layer),pi,wavel,refmed

      real s11,s12,s33,s34,se,sh,s11nor,smen,dang
      real qabp,qext,qsca,qback,g,ang,aj,pol,r,xa

      character*12 outf,intf
      character ss
      parameter (pi=3.14159265358979) 
      write(*,*)  ' _____________________________________________'
      write(*,*)  '    SCATTERING AND ABSORPTION EFFICIENCIES'         
      write(*,*)  '           FOR A MULTILAYERED SPHERE    ' 
      write(*,*)  '   '
      write(*,*)  '     This program calculates  scattering '
      write(*,*)  '  efficiencies for a multilayered sphere  '
      write(*,*)  '  with give refractive indices and radius '
      write(*,*)  '  within the various region of a multilayered'
      write(*,*)  '  sphere, refractive index of surrounding'
      write (*,*) '  medium, and free space wavelength. '
      write (*,*) ' _____________________________________________ '
      write (*,*) '   '
      write (*,*) '   '
      write (*,*) ' _______________________________________ '
      write (*,*) '    output data-file ?  (******.dat)'
      write (*,*) '   it includes all information about '
      write (*,*) '    input and output                 '        
      write (*,*) ' _______________________________________ '
      read (*,'(a)') outf
      open (3,file=outf)
      write (3,100)
c        refmed = (real) refractive index of surrouding medium
      write (*,*) ' _______________________________________________ '
      write (*,*) 'input real refractive index of surrouting medium'
      write (*,*) ' _______________________________________________ '
      read (*,*) refmed
      write (*,101) refmed
      write (3,101) refmed
      write (*,*) ' _____________________________________________ '
      write (*,*) 'input number of layers for multilayered sphere'
      write (*,*) '    m=  ?'
      write (*,*) ' _____________________________________________ '
      read (*,*) m
             if (m.gt.layer) then 
          write(*,*) ' number of layers greater then dim of array '
          write(*,*) ' set parameter (layer > ',m
      end if

      write (*,*) ' _____________________________________________ '
5     write (*,*) '  Would you input radii or size parameters'
      write (*,*) '   for multilayered sphere?  (r/s)'
      write (*,*) ' _____________________________________________ '
      read (*,'(a)') ss
      if (ss.eq.'r'.or.ss.eq.'R') goto 6
      if (ss.eq.'s'.or.ss.eq.'S') goto 7
      write (*,*) '    input error!'
      goto 5
6     write (*,*) ' ___________________________________ '
      write (*,*) '   input wavelength in free space'
      write (*,*) '____________________________________ '
      read (*,*) wavel
      write (*,102) wavel
      write (3,102) wavel
c     .......................................................
c     .  set the radii and the complex index of refraction  .
c     .    format :  real, (real,imag)                         .
c     .   ( note : imag. part is positive for absorption)   .
c     .......................................................
7     write (*,*) ' ___________________________________________ '
      write (*,*) '         input date-file ?  (******.dat)       '
      if (ss.eq.'r'.or.ss.eq.'R') then
      write (*,*) '  about radii (unit is same as wavelength)   ' 
      goto 8
      else
       write (*,*) '       about size parameters                     ' 
      end if
8     write (*,*) '       and complex refractive index            '
      write (*,*) '____________________________________________ '
      read (*,'(a)') intf
      open (1,file=intf,status='old')
       if (ss.eq.'r'.or.ss.eq.'R') then
      do 10 j=1,m
       read (1,*) r,rfrel(j)
       x(j)=2.*pi*r*refmed/wavel
       xa=sngl(x(j))
       write (*,103) j,r,xa,rfrel(j)
       write (3,103) j,r,xa,rfrel(j)
       rfrel(j)=rfrel(j)/refmed
10    continue
       else
       do 12 j=1,m
        read (1,*) x(j),rfrel(j)
        xa=sngl(x(j))
        write (*,104) j,xa,rfrel(j)
        write (3,104) j,xa,rfrel(j)
        rfrel(j)=rfrel(j)/refmed
12     continue
       end if
       close (1)
       yy=x(m)
c     .......................................................
c        calculate angular scattering intensity (Y)           .
c        only calculate scattering efficiencies Qe,Qs,Qa (N) .
c     .......................................................
45     write (*,*) ' _____________________________________________ '
       write (*,*) '    Would you calculate amplitude scattering'
       write (*,*) '    matrix elements?   (Y/N)'
       write (*,*) ' ____________________________________________ '
       read (*,'(a)') ss
       if (ss.eq.'Y'.or.ss.eq.'y') goto 46
       if (ss.eq.'N'.or.ss.eq.'n') goto 47
       write (*,*) '     input error'
       goto 45
46       write (*,*) ' _____________________________________________ '
       write (*,*) '   Please input NANG = number of angles 0 '
       write (*,*) '   and 90 degrees. The matrix elements'
       write (*,*) '     calculated at 2*NANG-1 angles inclu- '
       write (*,*) '     ding 0,90,and 180 degrees. When the'
       write (*,*) '    increament of angle is 1, NANG = 91.'
       write (*,*) '                 nang=?   ( nang>1 )'
       write (*,*) ' _____________________________________________ '
       read (*,*) nang

       if (nang.gt.winkel) then 
          write(*,*) ' number of degrees greater then dim of array '
          write(*,*) ' set parameter (layer > ',nang
       end if

       dang=1.570796327/float(nang-1)
       goto 48
47       nang=1
48       nan=2*nang-1

cd       allocate(s1(nan),s2(nan),stat=ie)

       call multibh(nang,nan,m,x,rfrel,qext,qsca,qback,
     *  g,s1,s2)
cd       if(ie.ne.0) goto 119
       qabp=qext-qsca
c     ...............................................................
c        output the efficiencies for scattering, extinction ,         .
c        absorption and backscattering, as well as asymmertry factor .
c     ...............................................................
       write (3,106)
       write (*,106)
       write (*,105) yy,qsca,qext,qabp,qback,g
       write (3,105) yy,qsca,qext,qabp,qback,g
       if (ss.eq.'n'.or.ss.eq.'N') goto 500
c     .......................................................
c        s11nor = s11 for 0 degree, i.e forward scattering
c        s1, s2 are scattering amplitude function
c     .......................................................
       s11nor=sngl(0.5*(cdabs(s2(1))*cdabs(s2(1))+cdabs(s1(1))*
     *           cdabs(s1(1))))
       write (3,112)
       write (3,113)
       write (3,114)
       write (3,113)
       write (3,107) s11nor 
       write (3,108)
       write (*,108)
c     ..............................................................
c        matrix elements normalized by s11, s11 is normalized        .
c        to 1.0 in forward direction, pol is degree of polarization .
c        (incident unpolarized plane wave)                              .
c     ..............................................................
       open (1,file='normal.dat')
       do 335 j=1,nan
       aj=dfloat(j)
       se=sngl(cdabs(s2(j))*cdabs(s2(j)))
       sh=sngl(cdabs(s1(j))*cdabs(s1(j)))
       s11=0.5*(se+sh)
       s12=0.5*(se-sh)
       pol=-s12/s11
       sri=s2(j)*dconjg(s1(j))
       s33=real(sri)/s11
       s34=sngl(dimag(sri)/s11)
       s11=s11/s11nor
cj       s11=alog10(s11)
       ang=dang*(aj-1.0)*57.2958
       write (3,109) ang,s11,pol,s33,s34
       write (1,109) ang,s11,pol,s33,s34
335     write (*,109) ang,s11,pol,s33,s34
       close (1)
       write (3,115)
       write (3,113)
       write (*,110)
       write (3,110)
c     ...........................
c        scattering intensities  .
c     ...........................
       open (4,file='inten.dat')
       do 70 j=1,nan
       aj=float(j)
       se=sngl(cdabs(s2(j))*cdabs(s2(j)))
       sh=sngl(cdabs(s1(j))*cdabs(s1(j)))
       s11=0.5*(se+sh)
       smen=s11
       ang=dang*(aj-1.0)*57.2958
cj       se=alog10(se)
cj       sh=alog10(sh)
cj       smen=alog10(smen)
       write (*,111) ang,se,sh,smen
       write (4,*) ang,se,sh,smen
70       write (3,111) ang,se,sh,smen
       close (4)
       close (3)
500     continue
100     format (/,9x,'SCATTERING AND ABSORPTION FOR MULTILAYERED',
     *         'SPHERE')
101     format (5x,'refmed=',e14.6)
102     format (5x,'wavelength=',f7.4)
103     format (5x,'j=',i3,3x,'r=',f8.3,3x,'x=',f8.3,3x,
     *          5x,'refra=',f10.6,1x,f10.6)
104     format (5x,'j=',i3,3x,'x=',f8.3,8x,'refra=',f10.6,1x,f10.6)
105     format (1x,f10.4,1x,e12.5,1x,e12.5,1x,e12.5,1x,e12.5,1x,e12.5)
106     format (/,6x,'y',9x,'qsca',9x,'qext',9x,'qabp',9x,'qback',10x,
     *         'g')
107     format (12x,'s11nor=',e13.6/10x,20('.'))
cj108     format (/,2x,'angle',4x,'Log(s11)',11x,'pol',13x,'s33',
108     format (/,2x,'angle',4x,'   (s11)',11x,'pol',13x,'s33',
     *         12x,'s34')
109     format (1x,f6.2,4(2x,e13.6))
cj110     format (/,2x,'angle',6x,'Log(spll)',6x,'Log(spdr)',
cj     *         6x,'Log(smen)')
110     format (/,2x,'angle',6x,'   (spll)',6x,'   (spdr)',
     *         6x,'   (smen)')
111     format (1x,f6.2,3(2x,e13.6))
112     format (/,26x,'STOKES PARAMETERS')
113     format (9x,55('='))
114     format (9x,'S33 and S34 matrix elements normalized by S11.',
     *         ' S11 is',/,9x,'normalized to 1.0 in the forward',
     *         ' direction. POL is ',/,9x,'degree polarization',
     *         ' for incident unpolarized light')
115     format (/,9x,'SPLL=S11+S12 SPDR=S11-S12 denote the incident',
     *         ' light',/,9x,'is polarized parallel and ',
     *         'perpendicular to scattering',/,9x,'plane',
     *         'respectively. SMEN=(SPLL+SPDR)/2 refers to ',/,9x,
     *         'incident unpolarized light.',/)
       end

c        multibh.for
c        .........................................................
c        .  Subroutine multibh calculates  extinction, scattering. 
c        .  backscattering efficiencies and asymmetry factor, as . 
c        .  well as scattering coefficients. And this calculates . 
c        .  also amplitude scattering matrix elements for given  . 
c        .  size parameters and refractive indices related to    . 
c        .  surrouting medeium within the various regions of the . 
c        .  multilayered sphere.                                 . 
c        .  ref.                                                 . 
c        .  Z. S. Wu and Y. P. Wang, Electromagnetic scattering  . 
c        .  for multilayered sphere: Recursive algorithms, Radio . 
c        .  Science, Vol.26, No. 6, p1393-1401, 1991.            . 
c        ......................................................... 
c        .  nang = control case, (=1): only calculate Qext, Qsca,. 
c        .         Qabp, Qback, g ( i.e.  qe, qs, qba, gg)       . 
c        .                          (>1): calculate also S1 and S2   
c        .                        and control increament of angle    
c        .  nan = number of scattered angle in 0 and 180         . 
c        .  m = number of layer                                  . 
c        .  x(j), y = size parameters in jth region and outer    . 
c        .              layer sphere related surrouting medium   . 
c        .  Let refel(j) = m(j),  complex refractive indies of   . 
c        .                            the jth region             . 
c        .    xj(j) = m(j)*x(j) ;  xj1(j) = m(j)*x(j-1)          . 
c        ......................................................... 
c        .  Ricatti-Bessel functions:  z*jn(z), z*yn(z), y*hn(y) .         
c        . the logarithmic derivatives of Ricatti-Bessel function. 
c        .   dp1x*[z] = [z*jn(z)]'/[z*jn(z)] = D(1)/n            . 
c        .   dch***[z] = [z*jn(z)]'/[z*yn(z)] = D(2)/n           . 
c        .   dxi**[y] = [y*hn(y)]'/[y*hn(y)] = D(3)/n            . 
c        .  where  z = xj(j) or xj1(j)                           . 
c        .     D(1)/n, 2-d array, downward recurrence            .
c        .     D(2)/n and Dn(3), 1-d array, upward recurrence    .
c        .   jn(z) = the spherical Bessel function               .
c        .   yn(z) = the spherical  Neumann  function            .
c        .   hn(y) = the first spherical Hankel function         .
c        .........................................................
c        .   the ratio of Ricatti-Bessel functions               .
c        .     pch***[z] = [z*jn(z)]/[z*yn(z)],  1-d array       .
c        .     pxiy = [y*jn(y)]/[y*hn(y)]                        .
c        .     chp*** = [z*yn(z)]/[z*jn(z)]                      .
c        .........................................................
c        .  an, bn = scattering coefficients                     .
c        .  s1[nn], s2[nn] = scattering amplitude functions      .
c        .  pi*[nn], tu[nn] = the anngle-dependent functions     .
c        .  amu[nn] = cos(theta) ,  (1-d array)                  .
c        .  theta = scattered angle, radian                      .
c        .........................................................
      subroutine multibh (nang,nn,m,x,rfrel,qext,
     *   qsca,qback,g,s1,s2)

       integer layer, winkel, groesse, j, i, ii, nang, n, nn,
     *           ns, nstop,m

      parameter( layer=100, winkel=200, groesse= 3000)

cd es gilt die abschaetzung :   ystop=2000+4.*2000**.3333+2. < 2500     

       real qext,qsca,qback,g,ystop,xmod
       
       double precision x(layer), y, dp1y(groesse), dch0y,
     *          dch1y,qs,qe,qba,gg,dang,pch0y,pch1y,
     *          amu(winkel),theta,pi(winkel),
     *          tau(winkel),pi0(winkel),p,
     *          pi1(winkel),ta,tb,rn,fn,tt
cd chp0y was soll diese var ? siehe zeile        
cd       complex san,sbn
       
       double complex s1(winkel),s2(winkel),a1,b1,t1,w1,xback,racap,
     *          q,racam,dxi0y,dxi1y,pxiy,z,an,bn,an1,bn1,
     *          xj1(layer),xj(layer),rfrel(layer),
     *          dp1x1(layer,groesse),dp1x(layer,groesse),
     *          pch0x1(winkel),pch1x1(winkel),
     *          pch0x(layer),pch1x(layer),
     *          dch0x1(layer),dch1x1(layer),
     *          dch0x(layer),dch1x(layer),rnc

c        .........................................
c        ouput scattering coefficients an and bn .
c        .........................................
      open (6,file='ab.dat')
       
       q=dcmplx(0.0,-1.0)

c        ..............................................................
c        calculate order term nstop, which is round-off, (ref. p1396) .
c        ..............................................................
       y=x(m)
       ystop=sngl(y+4.0d0*y**.3333+2.0d0)
       write (3,*) '   ystop=',ystop
       do 444 j=1,m
       xj(j)=rfrel(j)*x(j)
       xmod=sngl(cdabs(xj(j)))
444     ystop=amax1(xmod,ystop)
       nstop=int(ystop)
       ns=nstop+15
       write (3,*) '   xmod=',xmod
       write (3,*) '   n=   ',nstop,'     nmx=   ',ns
       write (*,*)'   n=   ',nstop,'     nmx=   ',ns

       if (nang.eq.1) goto 500
c        ................................
c        scattered angle and its cosine .
c        ................................
       dang=1.570796327/dfloat(nang-1)
       do 555 i=1,nang
       theta=(dfloat(i)-1.)*dang
555     amu(i)=dcos(theta)
c        .............................
c        Let initial values be zero  .
c        .............................
       qs=0.0d0
       qe=0.0d0
       gg=0.0d0
       xback=dcmplx(0.0d0,0.0d0)
       do 666 i=1,nang
       pi0(i)=0.0
666     pi1(i)=1.0d0
       nn=2*nang-1
       do 777 i=1,nn
       s1(i)=dcmplx(0.0d0,0.0d0)
777    s2(i)=dcmplx(0.0d0,0.0d0)
c        ......................................
c        series terminated after nstop terms  .
c        ......................................
500     continue
c        .........................................
c        calculate dp1y = [y*jn(y)]'/[y*jn(y)]   .
c        by downward recurrence formula, Eq.(33) .
c        .........................................
       call dprr(ns,y,dp1y)
c        .........................................................
c        initial values of logarithmic derivaties                  .
c        dch0y = D(1)/0(y)=-sin(y)/cos(y), dxi0y = D(3)/0(y) = i .
c        [y*y0(y)]/[y*j0(y)]=cos(y)/sin(y),  ref. Eq.(25)(27)(29).
c        .........................................................
       dch0y=-dsin(y)/dcos(y)
       pch0y=-dch0y
cd was soll das hier ? wird nirgendwo sonst noch benutzt !
cd       chp0y=dcos(y)/dsin(y)
       dxi0y=dcmplx(0.0,1.0)
c        .........................................................
c         if imag.(z) > 30. call subroutine bessel,                . 
c         calculate dch0x1=D(2)/0(z),z=m(j)*x(j-1)                 .
c         and dch0x(j)=D(2)/0(z), z=m(j)*x(j),  ref. Eq.(32)     .
c        .........................................................
       do 10 j=1,m
       if (j.eq.1) goto 15
       xj1(j)=rfrel(j)*x(j-1)
       z=xj1(j)
        if (dimag(z).gt.30.0d0) then
        call bessel (z,racap,racam)
       dch0x1(j)=q*racam/racap
       else
        dch0x1(j)=-cdsin(z)/cdcos(z)
       end if
       pch0x1(j)=-dch0x1(j)
15       xj(j)=rfrel(j)*x(j)
       z=xj(j)
       if (dimag(z).gt.30.0d0) then
           call bessel (z,racap,racam)
           dch0x(j)=q*racam/racap
         else
           dch0x(j)=-cdsin(z)/cdcos(z)
       end if
       pch0x(j)=-dch0x(j)
10       continue
c        .........................................................
c        the logarithmic derivaties of spherical Bessel function .
c        by downward recurrence, 2-d array, ref. Eq. (33)         .
c        dp1x1 = D(1)/n(z) = [z*jn(z)]'/[z*jn(z)], z=m(j)*x(j-1) .
c        dp1x = D(1)/n(z) = [z*jn(z)]'/[z*jn(z)], z=m(j)*x(j)    .                          .
c        .........................................................
       call dppp(m,ns,xj1,dp1x1,2)
cd       write (*,*) m,ns
       call dppp(m,ns,xj,dp1x,1)
cd       write (*,*) m,ns
c        .................................................
c        calculate scattering coefficients an and bn     .
c        h1 = Ha/n[m(j)*x(j)], w1 = Hb/n[m(j)*x(J)]       .
c        a1 = A(j)/n, b1 = B(j)/n,  ref. Eq. (18)-(21)   .
c        .................................................
       n=1
200    rn=dfloat(n)    
c        .................................................
c        D(2)/n(z), D(3)/n(z), [z*jn(z)]/[z*yn(z)] are   .
c        calculated by upward recurrence relations        .
c        z = y or m(j)*x(j), ref. Eq. (24), (28), (30)   .
c        .................................................
       rnc=dcmplx(rn,0.0d0)
       dch1y=dreal(1.d0/(rnc/y-dcmplx(dch0y,0.0d0))-rnc/y)
cd       dch1y=1.d0/(rnc/y-dch0y)-rnc/y
       dxi1y=1.d0/(rnc/y-dxi0y)-rnc/y
       pch1y=pch0y*dreal((dcmplx(dch1y,0.0d0)+rnc/y))
       pch1y=pch1y/dreal((dcmplx(dp1y(n))+rnc/y))
       pxiy=pch1y/(dcmplx(pch1y,0.0d0)+q)
       
       do 20 j=1,m
         dch1x(j)=1.0d0/(rn/xj(j)-dch0x(j))-rn/xj(j)
         pch1x(j)=(dch1x(j)+rn/xj(j))/(dp1x(j,n)+rn/xj(j))
         pch1x(j)=pch1x(j)*pch0x(j)
         if (j.eq.1.or.rfrel(j).eq.rfrel(j-1)) then
              a1=dcmplx(0.0d0,0.0d0)
              b1=dcmplx(0.0d0,0.0d0)
              t1=dp1x(j,n)
              w1=dp1x(j,n)
         else
              dch1x1(j)=1.0d0/(rn/xj1(j)-dch0x1(j))-rn/xj1(j)
              pch1x1(j)=(dch1x1(j)+rn/xj1(j))/(dp1x1(j,n)+rn/xj1(j))
              pch1x1(j)=pch1x1(j)*pch0x1(j)
              a1=rfrel(j)*t1-rfrel(j-1)*dp1x1(j,n)
              a1=a1/(rfrel(j)*t1-rfrel(j-1)*dch1x1(j))
              a1=a1*pch1x1(j)
              ta=cdabs(a1)
              b1=rfrel(j-1)*w1-rfrel(j)*dp1x1(j,n)
       	b1=b1/(rfrel(j-1)*w1-rfrel(j)*dch1x1(j))
              b1=b1*pch1x1(j)
              tb=cdabs(b1)
              if (ta.le.1.0d-30.and.tb.le.1.0d-30) then
       		 t1=dp1x(j,n)
       		w1=dp1x(j,n)
              else
       	  a1=a1/pch1x(j)
       		 t1=dp1x(j,n)-a1*dch1x(j)
       		 t1=t1/(dcmplx(1.0d0, 0.0d0)-a1)
       		 b1=b1/pch1x(j)
       		w1=dp1x(j,n)-b1*dch1x(j)
       		 w1=w1/(dcmplx(1.0d0,0.0d0)-b1)
              end if
       	  end if   
20       continue
c        .........................................................
c         output data about an and bn ,  ref.  Eq. (19) and (21) .
c        .........................................................
       an=t1-rfrel(m)*dp1y(n)
       an=an/(t1-rfrel(m)*dxi1y)
       bn=rfrel(m)*w1-dcmplx(dp1y(n),0.0d0)
       bn=bn/(rfrel(m)*w1-dxi1y)
       an=an*pxiy
       bn=bn*pxiy
cd es gibt keine typenwandlung von X*16 -> X*8, daher macht forchk hier zwei
cd fehler
cd       san=an
cd       sbn=bn
       write (6,100) n,an,bn
c        .................................................
c        calculate the efficiencies for scattering,       .
c        extinction, backscattering and asymmetry factor .
c        .................................................
       qs=qs+(2.0d0*rn+1.0d0)*(cdabs(an)*cdabs(an)+cdabs(bn)*cdabs(bn))
       xback=xback+(2.0d0*rn+1.0d0)*(an-bn)*(-1.0d0)**n
       qe=qe+(2.0d0*rn+1.0d0)*(dreal(an)+dreal(bn))
       if (n.eq.1) then 
         an1=an
         bn1=bn
       else
       gg=gg+4.0*((rn-1.0d0)/rn*(rn+1.0d0)*dreal(dconjg(an1)*an
     *  +bn1*dconjg(bn))+(2.0*rn-1.0d0)/rn/(rn-1.0d0)*
     *  dreal(dconjg(an1)*bn1))
       an1=an
       bn1=bn
       end if
       if (nang.eq.1) goto 780
c        .........................................
c        calculate angle-dependent function       .
c        .........................................
       fn=(2.0d0*rn+1.0d0)/(rn*(rn+1.0d0))
       do 789 i=1,nang
       ii=2*nang-i
       pi(i)=pi1(i)
       tau(i)=rn*amu(i)*pi(i)-(rn+1.0d0)*pi0(i)
       p=(-1.0d0)**(n-1)
       s1(i)=s1(i)+fn*(an*pi(i)+bn*tau(i))
       tt=(-1.0)**n
       s2(i)=s2(i)+fn*(an*tau(i)+bn*pi(i))
       if (i.eq.ii) goto 789
       s1(ii)=s1(ii)+fn*(an*pi(i)*p+bn*tau(i)*tt)
       s2(ii)=s2(ii)+fn*(an*tau(i)*tt+bn*pi(i)*p)
789    continue
780    dch0y=dch1y
       pch0y=pch1y
       dxi0y=dxi1y
       do 30 j=1,m
           if (j.eq.1) goto 16
            	pch0x1(j)=pch1x1(j)
       	     dch0x1(j)=dch1x1(j)
16         pch0x(j)=pch1x(j)
           dch0x(j)=dch1x(j)
30     continue
       n=n+1
       if (nang.eq.1) goto 900
       rn=dfloat(n)
       do 999 i=1,nang
       pi1(i)=((2.0d0*rn-1.0d0)/(rn-1.0d0))*amu(i)*pi(i)
       pi1(i)=pi1(i)-rn*pi0(i)/(rn-1.0d0)
999     pi0(i)=pi(i)
900     if (n-1-nstop) 200,300,300
300     qsca=sngl((2.0d0/(y*y))*qs)
       qext=sngl((2.0d0/(y*y))*qe)
       qba=cdabs(xback*dconjg(xback))
       qback=sngl((1.0d0/(y*y))*qba)
       g=sngl(gg/(2.0d0*qs))
100     format (2x,i4,4(1x,e13.6))
       return
       end

c        .................................................
c        the initial value of spherical Neumann function .
c        D(2)/0(z) = sin(z)/cos(z), except i in Eq.(32)  .
c        .................................................
       subroutine bessel (z,racap,racam)
       
       double precision x,y,xc,xs,yexp
       double complex z,racap,racam
       x=dreal(z)
       y=dimag(z)
       xc=dcos(2.0d0*x)
       xs=dsin(2.0d0*x)
       yexp=dexp(-2.0d0*y)
       racap=dcmplx(xc,xs)
       racam=dcmplx(1.0d0,0.0d0)-yexp*racap
       racap=dcmplx(1.0d0,0.0d0)+yexp*racap
       return
       end

c        .........................................................
c        calculate D(1)/n[z] by downward recurrence formula       .
c        nmx = nstop+15, z = m(j)*x(j) or m(j)*x(j-1)              .
c                          (ref. eq.(33))                              .
c        .........................................................
       subroutine dppp (m,nmax,z,dp,k)

       integer i,j,k,m,nmax,layer,groesse
       parameter (layer = 100, groesse = 3000)
       double complex z(layer),dp(layer,groesse)
       write(*,*) m,k
       do 100  j=k,m
       dp(j,nmax)=dcmplx(0.0d0,0.0d0)
       do 100 i=nmax-1,1,-1
100     dp(j,i)=(i+1)/z(j)-1.0d0/(dp(j,(i+1))+(i+1)/z(j))
       return
       end
       
c        .........................................................
c        calculate D(1)/n[z] by downward recurrence formula       .
c        nmx = nstop+15, z = y,                 (ref. eq.(33))     .
c        .........................................................
       subroutine dprr (nmax,z,dp)

       integer nmax,i,groesse
       parameter (groesse = 3000)
       double precision z,dp(groesse)
       dp(nmax)=0.0d0
       do 100 i=nmax-1,1,-1
100     dp(i)=(i+1)/z-1./(dp(i+1)+(i+1)/z)
       return
       end
